;***********************************
;*     IBM Keyboard Reader         *
;*                                 *
;* Ken Kaarvik    Mar30/99         *
;*                                 *
;***********************************

;Wiring info
;
;Connect your Color Gameboy to your AT keyboard
;
;GBC serial port   AT keyboard
;
;pin1 +5v          pin5 +5v
;pin2 Sout         nc
;pin3 Sin          pin2 Data
;pin4 (was P14)    pin1 Clk
;pin5 Clk          pin1 Clk
;pin6 0v           pin4 0v
;
;Pins 4&5 of the GBC serial port are connected
;together and connect to pin1 of the keyboard.
;
;Note : Print Screen & Pause aren't decoded properly
;


display_col		equ	$9809
display_quantity	equ	11
text_line		equ	$9800+$20*16

	INCLUDE	"hardware.inc"
	INCLUDE	"ibmpc1.inc"


        SECTION "Org $100",HOME[$100]

;*** Beginning of rom execution point ***

	nop
	jp	Startup

	NINTENDO_LOGO		;Nintendo graphic logo

;Rom Header Info

	;0123456789ABCDE
DB	"IBM keyboard   "	;Cart name 15bytes
DB	$80			;color gb
DB	0,0,0			;Not used
DB	CART_ROM		;Cart type
DB	CART_ROM_256K		;ROM Size (in bits)
DB	CART_RAM_NONE		;RAM Size (in bits)
DB	0,0			;Maker ID
DB	0			;Version     =0
DB	0			;Complement check (important)
DW	0			;Checksum (not important)



	INCLUDE	"memory1.asm"

	SECTION	"Main",home[$0150]

Startup
	call	initialize


Main

	call	get_raw_signal_ibm

	call	convert_to_ascii

	call	inc_counter

	;call	waitVB		;Do I really need this ???

	call	write_to_screen

	jp	Main

get_raw_signal_ibm
	ld	a,0
	ld	[ibm_raw_ir+1],a	;blank


wait_for_start_bit
	ldh	a,[$56]
	bit	4,a		;test pin 4 of GBC serial port
	jp	z,wait_for_start_bit

wait_for_start_bit_high
	ldh	a,[$56]
	bit	4,a
	jp	nz,wait_for_start_bit_high
	
wait_for_first_bit_low
	ldh	a,[$56]
	bit	4,a
	jp	z,wait_for_first_bit_low

	ld	a,$80
	ldh	[$02],a		;start serial xfer (external clock)

wait_for_ibm_byte

	ldh	a,[$02]
	bit	7,a
	jp	nz,wait_for_ibm_byte

	ldh	a,[$01]
	call	change_msb_order
	ld	a,b
	ld	[ibm_raw_ir],a

wait_for_stop_bit_clock_high
	ldh	a,[$56]
	bit	4,a
	jp	nz,wait_for_stop_bit_clock_high

	ld	a,[ibm_raw_ir]	;check for extended key
	cp	$E0
	ret	nz		;return from here if NOT extended key

wait_for_start_bit2
	ldh	a,[$56]
	bit	4,a		;test pin 4 of GBC serial port
	jp	z,wait_for_start_bit2

wait_for_start_bit_high2
	ldh	a,[$56]
	bit	4,a
	jp	nz,wait_for_start_bit_high2
	
wait_for_first_bit_low2
	ldh	a,[$56]
	bit	4,a
	jp	z,wait_for_first_bit_low2

	ld	a,$80
	ldh	[$02],a		;start serial xfer (external clock)

wait_for_ibm_byte2

	ldh	a,[$02]
	bit	7,a
	jp	nz,wait_for_ibm_byte2

	ldh	a,[$01]
	call	change_msb_order
	ld	a,b
	ld	[ibm_raw_ir+1],a
	ld	[ibm_key_value],a


wait_for_stop_bit_clock_high2
	ldh	a,[$56]
	bit	4,a
	jp	nz,wait_for_stop_bit_clock_high2

	ret

change_msb_order		;enter in a
	ld	b,$80		;leave in b
do_next_bit
	rlca
	rr	b
	ret	c
	jp	nz,do_next_bit


inc_counter
	ld	a,[counter]
	inc	a
	ld	[counter],a
	ret


write_to_screen
	call	shift_display_up

	call	shift_display_left

	ld	a,[ibm_ascii_value]
	ld	hl,text_line
	call	display_char


	ld	de,counter		;DISPLAY HERE #######
	ld	hl,display_col+9
	call	display_byte

	ld	de,ibm_raw_ir		;DISPLAY HERE #######
	ld	hl,display_col+6
	call	display_byte

	ld	de,ibm_raw_ir+1		;DISPLAY HERE #######
	ld	hl,display_col+3
	ld	a,[de]
	cp	0
	jp	z,show_0_as_blank

	call	display_byte
	ret
	
show_0_as_blank
	ld	a," "
	call	display_char
	inc	hl
	ld	a," "
	call	display_char

	ret

shift_display_left

	ld	a,[ibm_ascii_value_old]
	ld	b,a
	ld	a,[ibm_ascii_value]
	cp	b
	ret	z
	ld	[ibm_ascii_value_old],a

	ld	a,[ibm_ascii_value]

	bit	7,a		;check for non print char
	ret	nz

	ld	hl,text_line+18
	ld	de,text_line+19
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+17
	ld	de,text_line+18
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+16
	ld	de,text_line+17
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+15
	ld	de,text_line+16
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+14
	ld	de,text_line+15
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+13
	ld	de,text_line+14
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+12
	ld	de,text_line+13
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+11
	ld	de,text_line+12
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+10
	ld	de,text_line+11
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+9
	ld	de,text_line+10
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+8
	ld	de,text_line+9
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+7
	ld	de,text_line+8
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+6
	ld	de,text_line+7
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+5
	ld	de,text_line+6
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+4
	ld	de,text_line+5
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+3
	ld	de,text_line+4
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+2
	ld	de,text_line+3
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line+1
	ld	de,text_line+2
	ld	bc,1
	call	mem_CopyVRAM

	ld	hl,text_line
	ld	de,text_line+1
	ld	bc,1
	call	mem_CopyVRAM

	ret

;shift_display_up

	ld	hl,display_col+$20*17	;copy from
	ld	de,display_col+$20*18	;copy to
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*16
	ld	de,display_col+$20*17
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*15
	ld	de,display_col+$20*16
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*14
	ld	de,display_col+$20*15
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*13
	ld	de,display_col+$20*14
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*12
	ld	de,display_col+$20*13
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*11
	ld	de,display_col+$20*12
	ld	bc,display_quantity
	call	mem_CopyVRAM
shift_display_up
	ld	hl,display_col+$20*10
	ld	de,display_col+$20*11
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*9
	ld	de,display_col+$20*10
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*8
	ld	de,display_col+$20*9
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*7
	ld	de,display_col+$20*8
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*6
	ld	de,display_col+$20*7
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*5
	ld	de,display_col+$20*6
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*4
	ld	de,display_col+$20*5
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+$20*3
	ld	de,display_col+$20*4
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+($20*2)
	ld	de,display_col+($20*3)
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+($20*1)
	ld	de,display_col+($20*2)
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ld	hl,display_col+($20*0)
	ld	de,display_col+($20*1)
	ld	bc,display_quantity
	call	mem_CopyVRAM

	ret

display_byte
	lcd_WaitVRAM
	ld	a,[de]
	ld	b,a
	and	$0F
	ld	[hl+],a
	lcd_WaitVRAM
	ld	a,b
	and	$F0
	swap	a
	ld	[hl],a
	ret

display_char
	bit	7,a		;don't look up non print chars
	ret	nz

	add	a,20		;add imb char offset
	push	af
	lcd_WaitVRAM
	pop	af
	ld	[hl],a
	ret

waitVB
	ldh	a,[$44]
	cp	144
	jp	nz,waitVB
	ret

initialize

.wait	ldh	a,[$44]		;LY LCDC compare
	cp	144
	jr	nc,.wait
	ld	a,0
	ldh	[$40],a		;LCDC lcd control

	ld	a,%10000000
	ldh	[$68],a		;BCPS
	ld	a,%00000000	;palette 0 0  bg
	ldh	[$69],a		;BCPD
	ld	a,%00000000
	ldh	[$69],a

	ld	a,%11111110	;palette 0 1
	ldh	[$69],a
	ld	a,%00011110
	ldh	[$69],a

	ld	a,%11111111	;palette 0 2  fg test font
	ldh	[$69],a
	ld	a,%01111111
	ldh	[$69],a

	ld	a,%11111111	;palette 0 3  fg ibm font
	ldh	[$69],a
	ld	a,%01111111
	ldh	[$69],a

	;ld	a,%00000000	;palette 1 0  bg
	;ldh	[$69],a
	;ld	a,%00000000
	;ldh	[$69],a

	ld	hl,Font		;load my test Font
	ld	de,$8000
	ld	bc,20*8*2
	call	mem_Copy

	ld	hl,ibm_characters	;load ibm font
	ld	de,$8000+20*8*2		;$8140
	ld	bc,8*128
	call	mem_CopyMono

	ld	a,1
	ldh	[$4F],a		;VBK
	ld	a,%01100000	;flip my screen
	ld	hl,$9800
	ld	bc,32*32
	call	mem_Set
	ld	a,0
	ldh	[$4F],a		;VBK


	ld	a,16		;blank char
	ld	hl,$9800
	ld	bc,20*32*32
	call	mem_Set

	ld	a,0
	ldh	[$42],a		;SCY Scroll Y
	ldh	[$43],a		;SCX Scroll X

	ld	a,%00000011
	ldh	[$47],a		;BGP

	ld	a,$00000000
	ldh	[$FF],a		;IE

	ld	a,%10010001
	ldh	[$40],a

	xor	a
	ld	[$FF24],a

	xor	a
	ld	[counter],a
	ld	[ibm_raw_ir],a

	ld	a,%11000000
	ldh	[$56],a


	ret


convert_to_ascii
	ld	hl,ibm_table
	ld	b,0
	ld	a,[ibm_raw_ir]
	bit	7,a			;check for key up 
	jp	nz,ascii_key_up
	ld	c,a	
	add	hl,bc
	ld	a,[hl]

	ld	[ibm_ascii_value],a
	ret

ascii_key_up
	ld	a,$FF			;
	ld	[ibm_ascii_value],a	;
	ret                                                               

ibm_characters
	chr_IBMPC1	1,3

ibm_table
	DB	$00	;	$00
	DB	$00	;F9	01
	DB	$00	;	02
	DB	$00	;F5	03
	DB	$00	;	04
	DB	$00	;	05
	DB	$00	;	06
	DB	$00	;	07
	DB	$00	;	08
	DB	$00	;	09
	DB	$00	;	0A
	DB	$00	;	0B
	DB	$00	;	0C
	DB	$00	;	0D
	DB	$00	;	0E
	DB	$00	;	0F
	DB	$00	;	10
	DB	$00	;	11
	DB	$00	;	12
	DB	$00	;	13
	DB	$00	;	14
	DB	$51	;Q	15
	DB	$31	;1	16
	DB	$00	;	17
	DB	$00	;	18
	DB	$00	;	19
	DB	$5A	;Z	1A
	DB	$53	;S	1B
	DB	$41	;A	1C
	DB	$57	;W	1D
	DB	$32	;2	1E
	DB	$00	;	1F
	DB	$00	;	20
	DB	$43	;C	21
	DB	$58	;X	22
	DB	$44	;D	23
	DB	$45	;E	24
	DB	$34	;4	25
	DB	$33	;3	26
	DB	$00	;	27
	DB	$00	;	28
	DB	$20	;Space	29
	DB	$56	;V	2A
	DB	$46	;F	2B
	DB	$54	;T	2C
	DB	$52	;R	2D
	DB	$35	;5	2E
	DB	$00	;	2F
	DB	$00	;	30
	DB	$4E	;N	31
	DB	$42	;B	32
	DB	$48	;H	33
	DB	$47	;G	34
	DB	$59	;Y	35
	DB	$36	;6	36
	DB	$00	;	37
	DB	$00	;	38
	DB	$00	;	39
	DB	$4D	;M	3A
	DB	$4A	;J	3B
	DB	$55	;U	3C
	DB	$37	;7	3D
	DB	$38	;8	3E
	DB	$00	;	3F
	DB	$00	;	40
	DB	$2C	;,	41
	DB	$4B	;K	42
	DB	$49	;I	43
	DB	$4F	;O	44
	DB	$30	;0	45
	DB	$39	;9	46
	DB	$00	;	47
	DB	$00	;	48
	DB	$2E	;.	49
	DB	$2F	;/	4A
	DB	$4C	;L	4B
	DB	$3B	;;	4C
	DB	$50	;P	4D
	DB	$2D	;-	4E
	DB	$00	;	4F
	DB	$00	;	50
	DB	$00	;	51
	DB	$27	;'	52
	DB	$00	;	53
	DB	$5B	;[	54
	DB	$3D	;=	55
	DB	$00	;	56
	DB	$00	;	57
	DB	$00	;	58
	DB	$00	;	59
	DB	$20	;Enter	5A
	DB	$5D	;]	5B
	DB	$00	;	5C
	DB	$5C	;\	5D
	DB	$00	;	5E
	DB	$00	;	5F
	DB	$00	;	60
	DB	$00	;	61
	DB	$00	;	62
	DB	$00	;	63
	DB	$00	;	64
	DB	$00	;	65
	DB	$00	;	66
	DB	$00	;	67
	DB	$00	;	68
	DB	$31	;num1	69
	DB	$00	;	6A
	DB	$34	;num4	6B
	DB	$37	;num7	6C
	DB	$00	;	6D
	DB	$00	;	6E
	DB	$00	;	6F
	DB	$30	;num0	70
	DB	$2E	;num.	71
	DB	$32	;num2	72
	DB	$35	;num5	73
	DB	$36	;num6	74
	DB	$38	;num8	75
	DB	$00	;	76
	DB	$00	;	77
	DB	$00	;	78
	DB	$2B	;num+	79
	DB	$33	;num3	7A
	DB	$2D	;num-	7B
	DB	$2A	;num*	7C
	DB	$39	;num9	7D
	DB	$00	;	7E
	DB	$00	;	7F
Font:
	DW	`01111100
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`01111100
	DW	`00000000

	DW	`00010000
	DW	`00110000
	DW	`00010000
	DW	`00010000
	DW	`00010000
	DW	`00010000
	DW	`00111000
	DW	`00000000

	DW	`01111100
	DW	`10000010
	DW	`00000010
	DW	`01111100
	DW	`10000000
	DW	`10000000
	DW	`11111110
	DW	`00000000

	DW	`01111100
	DW	`10000010
	DW	`00000010
	DW	`00011100
	DW	`00000010
	DW	`10000010
	DW	`01111100
	DW	`00000000

	DW	`00001100
	DW	`00010100
	DW	`00100100
	DW	`01000100
	DW	`11111110
	DW	`00000100
	DW	`00000100
	DW	`00000000

	DW	`11111110
	DW	`10000000
	DW	`10000000
	DW	`11111100
	DW	`00000010
	DW	`00000010
	DW	`11111100
	DW	`00000000

	DW	`01111100
	DW	`10000000
	DW	`10000000
	DW	`11111100
	DW	`10000010
	DW	`10000010
	DW	`01111100
	DW	`00000000

	DW	`11111110
	DW	`00000010
	DW	`00000100
	DW	`00001000
	DW	`00010000
	DW	`00010000
	DW	`00010000
	DW	`00000000

	DW	`01111100
	DW	`10000010
	DW	`10000010
	DW	`01111100
	DW	`10000010
	DW	`10000010
	DW	`01111100
	DW	`00000000

	DW	`01111100
	DW	`10000010
	DW	`10000010
	DW	`01111110
	DW	`00000010
	DW	`00000010
	DW	`01111100
	DW	`00000000

	DW	`00111000
	DW	`01000100
	DW	`10000010
	DW	`11111110
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`00000000

	DW	`11111100
	DW	`10000010
	DW	`10000010
	DW	`11111100
	DW	`10000010
	DW	`10000010
	DW	`11111100
	DW	`00000000

	DW	`00111100
	DW	`01000010
	DW	`10000000
	DW	`10000000
	DW	`10000000
	DW	`01000010
	DW	`00111100
	DW	`00000000

	DW	`11111000
	DW	`10000100
	DW	`10000010
	DW	`10000010
	DW	`10000010
	DW	`10000100
	DW	`11111000
	DW	`00000000

	DW	`11111110
	DW	`10000000
	DW	`10000000
	DW	`11111000
	DW	`10000000
	DW	`10000000
	DW	`11111110
	DW	`00000000

	DW	`11111110
	DW	`10000000
	DW	`10000000
	DW	`11111000
	DW	`10000000
	DW	`10000000
	DW	`10000000
	DW	`00000000

	DW	`00000000	;16
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000

	DW	`11111111	;17 web mark
	DW	`10000001
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000

	DW	`00000000	;18 web space
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`00000000
	DW	`10000001
	DW	`11111111
	DW	`00000000

	DW	`11111100	;19
	DW	`10000010	;P for parity
	DW	`10000010
	DW	`11111100
	DW	`10000000
	DW	`10000000
	DW	`10000000
	DW	`00000000


	SECTION	"GB_ram",BSS
counter			DS	1
ibm_key_value		DS	1
ibm_ascii_value		DS	1
ibm_ascii_value_old	DS	1
ibm_char_value		DS	1
ibm_raw_ir		DS	8